/**
 *  @file    instr.c
 *  @author  Guillaume Bour. 2002
 
 *  @version 0.1
 *    @date  01/02/2002

 *  instructions generation
 */
/*      Copyright (C) 2002 Guillaume Bour
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *  
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *  
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 59 Temple Place - Suite 330, Boston, 
 *      MA 02111-1307, USA.
 */
#include <stdio.h>
#include <gtk/gtk.h>
#include "instr.h"
#include "type.h"
#include "op.h"
#include "middle-code.h"
#include "debug.h"

#ifdef DEBUG1
  #include "turtle.h"
  #include "variable.h"
  extern struct s_turtle my_turtle;

gchar *op_str[44] = {
  "OP_NOOP",
  "OP_PROCCALL",
  "OP_RETURN",
  "OP_MOVEFORWARD",
  "OP_MOVEBACK",
  "OP_TURNRIGHT",
  "OP_TURNLEFT",
  "OP_SETPOS",
  "OP_SETHEADING",
  "OP_CLEARSCREEN",
  "OP_HIDETURTLE",
  "OP_SHOWTURTLE",
  "OP_HIDEPEN",
  "OP_SHOWPEN",
  "OP_SETPREDEFCOLOR",
  "OP_SETUSERDEFCOLOR",
  "OP_NOTRACE",
  "OP_TRACE",
  "OP_WRITE",
  "OP_STOP",
  "OP_AFFECT",
  "OP_LOCAFFECT",
  "OP_DANCE",
  "OP_SLEEP",
  "OP_GETHEADING",
  "OP_HAZARD",
  "OP_REPEAT",
  "OP_LOOPCNT",
  "OP_IF",
  "OP_ELSE",
  "OP_WHILE",
  "OP_JUMP",
  "OP_AND",
  "OP_OR",
  "OP_EQ",
  "OP_LT",
  "OP_GT",
  "OP_LEQ",
  "OP_GEQ",
  "OP_ADD",
  "OP_SUB",
  "OP_MUL",
  "OP_DIV",
  "OP_UNEG"
};
#endif

/**
   Instruction allocation.<br>

   <b>visibility :: <i>public</i></b>

   @remarks
            1.the instruction current value is set to <null>

   @param   op                          the instruction operator(ADD, SUB, ...)
   @param   type                        the instruction type(int, bool, ...)
   @param   arg1                        the first argument
   @param   arg2                        the second argument

   @return  the allocated instruction
 */
struct s_instr *instr_alloc(op_t op, type_t type, 
			    struct s_mc *arg1, struct s_mc *arg2)
{
  struct s_instr *myinstr = g_new(struct s_instr, 1);

  myinstr->type = type;
  myinstr->op = op;

  myinstr->arg1 = arg1;
  if(arg1 != NULL) 
    { arg1->refcnt++; }
  myinstr->arg2 = arg2;
  if(arg2 != NULL) 
    { arg2->refcnt++; }

  myinstr->value = NULL;

  return(myinstr);
}


/**
   Instruction freeing.<br>

   <b>visibility :: <i>public</i></b>

   @remarks
            1. the second argument is not used (just for GTK compatibility)

   @param   elt                         the instruction to free
   @param   nil                         not used
   @return  <none>
 */
void instr_free(gpointer elt, gpointer nil)
{
  register struct s_instr *instr =(struct s_instr *)elt;

  mc_free(instr->arg1);
  mc_free(instr->arg2);

  g_free(instr->value);
  g_free(instr);
}


#ifdef DEBUG1
void instr_debug(gpointer elt, gpointer dec)
{
  gint pos;
  gchar txt[100], 
    arg1[15] = "<null>", arg2[15] = "<null>";
  register struct s_instr *instr =(struct s_instr *)elt;

  if(instr->arg1 != NULL && instr->arg1->arg != NULL)
    {
      switch(instr->arg1->t_arg)
	{
	case MC_VAR:
	  g_snprintf(arg1, 15, "*%d:%s", instr->arg1->arg,
		     ((struct s_variable *)instr->arg1->arg)->name); 
	  break;
	case MC_LIT:
	  g_snprintf(arg1, 15, "%s", 
		     lit_debug((struct s_literal *)instr->arg1->arg));
	  break;
	case MC_REF: 
	  g_snprintf(arg1, 15, "*%d", instr->arg1->arg);
	  break;
	case MC_LIST: 
	  g_snprintf(arg1, 15, "<list>");
	}
    }

  if(instr->arg2 != NULL && instr->arg2->arg != NULL)
    {
      switch(instr->arg2->t_arg)
	{
	case MC_VAR:
	  g_snprintf(arg2, 15, "*%d:%s", instr->arg2->arg,
		     ((struct s_variable *)instr->arg2->arg)->name); 
	  break;
	case MC_LIT:
	  g_snprintf(arg2, 15, "%s", 
		     lit_debug((struct s_literal *)instr->arg2->arg));
	  break;
	case MC_REF: 
	  g_snprintf(arg2, 15, "*%d", instr->arg2->arg);
	  break;
	case MC_LIST: 
	  g_snprintf(arg2, 15, "<list>");
	}
    }

  debug_append2("    (*%d: %s [%s, %s])\n", instr, op_str[instr->op], 
		arg1, arg2);
}
#endif
