/**
 *  @file    literal.c
 *  @author  Guillaume Bour. 2002
 
 *  @version 0.1
 *    @date  10/02/2002

 *  literals management
 */
/*      Copyright (C) 2002 Guillaume Bour
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *  
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *  
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 59 Temple Place - Suite 330, Boston, 
 *      MA 02111-1307, USA.
 */
#include <stdio.h>
#include <stdarg.h>
#include <gtk/gtk.h>
#include "literal.h"


/**
   Literal allocation.<br>

   <b>visibility :: <i>public</i></b>

   @param   type                        the literal type(int, bool, ...)
   @param   value                       the literal value(3, TRUE, ...)

   @return  the allocated literal
 */
struct s_literal *lit_alloc(type_t type, ...)
{
  va_list ap;
  struct s_literal *mylit = g_new(struct s_literal, 1);

  mylit->type = type;

  va_start(ap, type);
  switch(type)
    {
    case T_INT   : 
      {
	guint value = va_arg(ap, guint);
	mylit->value = g_memdup(&value, sizeof(guint));
	break;
      }
    case T_FLOAT : 
      {
	gdouble value = va_arg(ap, gdouble);
	mylit->value = g_memdup(&value, sizeof(gdouble));
	break;
      }
    case T_BOOL  : 
      {
	gboolean value = va_arg(ap, gboolean);
	mylit->value = g_memdup(&value, sizeof(gboolean));
	break;
      }
    case T_STRING: 
      {
	gchar *value = va_arg(ap, gchar *);
	mylit->value = 
	  g_memdup(value, sizeof(gchar) * strlen((gchar *)value)+1);

	printf("bef: %s, aft: %s\n", value, (gchar *)mylit->value);
	break;
      }
    default      : mylit->value = NULL;
    }
  va_end(ap);
  
  return(mylit);
}


/**
   Literal freeing.<br>

   <b>visibility :: <i>public</i></b>

   @remarks
            1. the second argument is not used (just for GTK compatibility)

   @param   elt                         the literal to free
   @param   nil                         not used
   @return  <none>
 */
void lit_free(gpointer elt, gpointer nil)
{
  register struct s_literal *lit =(struct s_literal *)elt;

  g_free(lit->value);
  g_free(lit);
}


#ifdef DEBUG1
gchar *lit_debug(struct s_literal *lit)
{
  gchar *litstr = g_new(gchar, 10);

  switch(lit->type)
    {
    case T_INT   : 
      g_snprintf(litstr, 10, "%d", *(guint *)lit->value);
      break;
    case T_FLOAT : 
      g_snprintf(litstr, 10, "%f", *(gdouble *)lit->value);
      break;
    case T_BOOL  : 
      g_snprintf(litstr, 10, "%s", 
		 *(gboolean *)lit->value == TRUE?"<true>":"false");
      break;
    case T_STRING: 
      g_snprintf(litstr, 10, "\"%s\"", (gchar *)lit->value);
      break;
    default      : 
      g_free(litstr);
      litstr = NULL;
    }

  return(litstr);
}
#endif
