/**
 *  @file    type.c
 *  @author  Guillaume Bour. 2002
 
 *  @version 0.1
 *    @date  09/02/2002

 *  type checking/variables type inference.
 */
#include <stdio.h>
#include <stdarg.h>
#include "type.h"
#include "op.h"

/* tables de cohrence des oprations entre types
 *
 * exemple: l'opration 'int + boolean' est-elle valide: NON
 *                              (retourne un type T_ILLEGAL)
 */
type_t type_coherence[5][4][4] = {
  {
  /* oprations arithmtiques (sauf l'addition)             */
  /*               integer    float      boolean    string  */
  /* integer */ { T_INT    , T_FLOAT  , T_ILLEGAL, T_ILLEGAL },
  /* float   */ { T_FLOAT  , T_FLOAT  , T_ILLEGAL, T_ILLEGAL },
  /* boolean */ { T_ILLEGAL, T_ILLEGAL, T_ILLEGAL, T_ILLEGAL },
  /* string  */ { T_ILLEGAL, T_ILLEGAL, T_ILLEGAL, T_ILLEGAL }
  },{

  /* opration d'addition                                   */
  /*               integer    float      boolean    string  */
  /* integer */ { T_INT    , T_FLOAT  , T_ILLEGAL, T_ILLEGAL },
  /* float   */ { T_FLOAT  , T_FLOAT  , T_ILLEGAL, T_ILLEGAL },
  /* boolean */ { T_ILLEGAL, T_ILLEGAL, T_ILLEGAL, T_ILLEGAL },
  /* string  */ { T_ILLEGAL, T_ILLEGAL, T_ILLEGAL, T_STRING  }
  },{

  /* oprations logiques (sauf l'galit)                   */
  /*               integer    float      boolean    string  */
  /* integer */ { T_ILLEGAL, T_ILLEGAL, T_ILLEGAL, T_ILLEGAL },
  /* float   */ { T_ILLEGAL, T_ILLEGAL, T_ILLEGAL, T_ILLEGAL },
  /* boolean */ { T_ILLEGAL, T_ILLEGAL, T_BOOL   , T_ILLEGAL },
  /* string  */ { T_ILLEGAL, T_ILLEGAL, T_ILLEGAL, T_ILLEGAL }
  },{
 
  /* opration d'galit                                    */
  /*               integer    float      boolean    string  */
  /* integer */ { T_BOOL   , T_BOOL   , T_ILLEGAL, T_ILLEGAL },
  /* float   */ { T_BOOL   , T_BOOL   , T_ILLEGAL, T_ILLEGAL },
  /* boolean */ { T_ILLEGAL, T_ILLEGAL, T_BOOL   , T_ILLEGAL },
  /* string  */ { T_ILLEGAL, T_ILLEGAL, T_ILLEGAL, T_BOOL    }
  },{

  /* oprations de comparaisons                             */
  /*               integer    float      boolean    string  */
  /* integer */ { T_BOOL   , T_BOOL   , T_ILLEGAL, T_ILLEGAL },
  /* float   */ { T_BOOL   , T_BOOL   , T_ILLEGAL, T_ILLEGAL },
  /* boolean */ { T_ILLEGAL, T_ILLEGAL, T_ILLEGAL, T_ILLEGAL },
  /* string  */ { T_ILLEGAL, T_ILLEGAL, T_ILLEGAL, T_BOOL    }
  }

  /* actions  1 argument(entier ou flottant)               */
  /*               integer    float      boolean    string  */
  /* integer */ { T_NONE   , T_BOOL   , T_ILLEGAL, T_ILLEGAL },
  /* float   */ { T_BOOL   , T_BOOL   , T_ILLEGAL, T_ILLEGAL },
  /* boolean */ { T_ILLEGAL, T_ILLEGAL, T_ILLEGAL, T_ILLEGAL },
  /* string  */ { T_ILLEGAL, T_ILLEGAL, T_ILLEGAL, T_BOOL    }
  }
};

/* table de cohrence correspondans  chaque oprateur.
 *
 * exemple: pour l'oprateur 'not' (non boolen), on utilise la table
 *          logic_coherence (indice 2)
 */
int op_coherence[13] = {
  /* negation       */ 0,
  /* addition       */ 1,
  /* substraction   */ 0,
  /* multiplication */ 0,
  /* division       */ 0,
  
  /* not            */ 2,
  /* or             */ 2,
  /* and            */ 2,
  
  /* eq             */ 3,
  /* lt             */ 4,
  /* gt             */ 4,
  /* leq            */ 4,
  /* geq            */ 4
};




/*
 * Vrifie la cohrence d'une opration 
 *
 * entres:
 *   + un oprateur
 *   + le type du premier argument 
 *   + le type du second argument    (ommis dans le cas d'une opration unaire)
 *
 * sortie : le type rsultant de l'opration 
 *          (T_INVALID si l'opration est invalide)
 */
/*type_t type_is_coherent(op_t op, type_t arg1, ...)
{
  va_list ap;
  type_t arg2;

  *//* rcupration du second argument *//*
  if(op == OP_NEG)
    { arg2 = T_INT; }
  else if(op == OP_NOT)
    { arg2 = T_BOOL; }
  else
    {
      va_start(ap, arg1);
      arg2 = va_arg(ap, type_t);
      va_end (ap);
    }

  return(type_coherence[op_coherence[op]][arg1][arg2]);
}*/
type_t type_is_coherent(op_t op, ...)
{
  return(T_NONE);
}
